<?php
/*
Plugin Name: LazyCaptcha
Description: LazyCaptcha is a small and lazy plugin to prevent bots from spamming your comments.
Version: 0.3
Author: Hans Matzen
Author URI: https://www.tuxlog.de
License:     GPL2
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Domain Path: lang
Text Domain: wp-lazycaptcha
*/

/*  
    Copyright 2020-2022 - Hans Matzen 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// plugin activation function
function wp_lazy_captcha_activate() {
    global $wpdb;

    // check for db-table and create if necessary
	$sql = 'SHOW TABLES LIKE \''.$wpdb->prefix.'lazy_captcha\'';

	if($wpdb->get_var($sql) != $wpdb->prefix.'lazy_captcha')
	{
		// create challenge table
		$sql = "create table ".$wpdb->prefix."lazy_captcha
		(
		nid varchar(32) NOT NULL,
		task varchar(12) NOT NULL,
		result varchar(3) NOT NULL,
		chtime timestamp NOT NULL,
		primary key(nid)
		)";
		$results = $wpdb->query($sql);
    }
}

// plugin deactivation function
function wp_lazy_captcha_deactivate() {
    global $wpdb;

    // check for db-table and delete if applicable
	$sql = 'SHOW TABLES LIKE \''.$wpdb->prefix.'lazy_captcha\'';
	if($wpdb->get_var($sql) == $wpdb->prefix.'lazy_captcha')
	{
		// drop table
		$sql = "drop table ".$wpdb->prefix."lazy_captcha;";
		$results = $wpdb->query($sql);
	}
    
    // remove cron job for db-cleanup
    wp_clear_scheduled_hook( 'wp_lazy_captcha_dbclean_event' );
}

// plugin init function
function wp_lazy_captcha_init()
{
    // get translation 
    load_plugin_textdomain('wp-lazycaptcha', false, dirname( plugin_basename( __FILE__ ) ) . "/lang/");   
    
    wp_register_style( 'wplc',  plugins_url('wplc.css',__FILE__ ) );
    wp_enqueue_style('wplc'); 
    
    // add wp-cron event to regulary remove old entries in the db-table
    if ( ! wp_next_scheduled( 'wp_lazy_captcha_dbclean_event' ) ) {
        wp_schedule_event( time(), 'twicedaily', 'wp_lazy_captcha_dbclean_event' );
    }
}

// clean up database and remove all records older than 12 hours
function wp_lazy_captcha_dbclean() {
    global $wpdb;
    $sql = "delete from " . $wpdb->prefix . "lazy_captcha where chtime < DATE_SUB(CURRENT_TIMESTAMP, INTERVAL 12 HOUR);";
    $results = $wpdb->query($sql);
}
add_action( 'wp_lazy_captcha_dbclean_event', 'wp_lazy_captcha_dbclean' );

// show captcha task between textarea and submit button
function wp_lazy_captcha_comment_meta( $defaults ) {
	//$defaults['comment_notes_after'] .= wp_lazy_captcha_show();
    $defaults['fields']['wp_lazy_captcha'] = wp_lazy_captcha_show();
    
	return $defaults;
}

function wp_lazy_captcha_store($nid, $task, $result) {
    global $wpdb;

	$wpdb->insert( $wpdb->prefix . "lazy_captcha", 
		array( 
			'nid'    => $nid, 
			'task'   => $task, 
			'result' => $result, 
			'chtime' => current_time('mysql'),
		)
	);
}
// create html to show captcha and return it
function wp_lazy_captcha_show() {
    list($task, $result) = wp_lazy_captcha_generate_task();
    $nid = "wplc_" . uniqid();
    
    // save task, result and id to database
    wp_lazy_captcha_store($nid, $task, $result);
    
    // create a 100*30 image
    $im = imagecreate(100, 30);

    // Weißer Hintergrund und blauer Text
    $bg = imagecolorallocate($im, 255, 255, 255);
    $textcolor = imagecolorallocate($im, 44, 44, 44);

    // add text to image
    $font = plugin_dir_path(__FILE__) . 'junkos/junkos_typewriter.ttf';
        
    imagettftext($im, 14, 0, 10, 20, $textcolor, $font, $task);
    
    // capture image file as string
    ob_start();
    imagepng($im);
    $image_data = ob_get_contents();
    ob_end_clean();
          
    // Read image path, convert to base64 encoding
    $imageData = base64_encode($image_data);
    
    // Format the image SRC:  data:{mime};base64,{data};
    $src = 'data: '.'image/png'.';base64,'.$imageData;

    // Echo out a sample image
    $html  = '<p class="comment-form-author"><label for="'.$nid.'">' . __( 'Enter Result', 'wp-lazycaptcha' ) . '<span class="required">*</span></label><br/>';
    $html .= '<img src="' . $src . '" width="200" height="45" class="imgwplc">';
    $html .= '<input id="authorwplc" name="' . $nid . '" type="text" value="" maxlength="2" size="5">';
    $html .= "</p>";

    imagedestroy($im);
    
    return $html;
}

// generate random arithmetic task and return it together with the result as a tuple
function wp_lazy_captcha_generate_task() {
    $op0 = wp_rand(0,1);
    $op  = ($op0 === 1 ? "+" : "-");
    $op1 = wp_rand(0,20);
    $op2 = wp_rand(1,10);
   
   // avoid negative and zero results results
   if ("-" === $op and $op2 >= $op1) {
       $t1 = $op2; 
       $op2 = $op1; 
       $op1 = $t1; 
   }
   
    if ( "+" === $op )
        $result = $op1 + $op2;
    else
        $result = $op1 -$op2;

    $task = $op1 . " " . $op . " " . $op2 . " =";
    
    return array($task ,$result);
}

	
// check if result is correct and not empty
function wp_lazy_captcha_check($comment) {
    // do nothing if user is logged in or we got a ping or trackback    
    if ( is_user_logged_in() ) return $comment;
    if ( $comment['comment_type'] != "comment" ) return $comment;
        
    global $wpdb;
    
	// pre filter $_POST to only get keys starting with wplc_ 
    $hit = array_filter( $_POST, function ( $key) { return substr($key, 0, 5) === "wplc_"; }, ARRAY_FILTER_USE_KEY );
    
    // field is not available, stop it here
    if ( empty($hit) ) {
		wp_die( 
			apply_filters('pre_comment_content', 
			"<strong>" . __( 'Error', 'wp-lazycaptcha' ) . '</strong>:' . __( 'Please fill in the correct answer and try again.', 'wp-lazycaptcha' ) ),
			'Error',
			array('back_link' => true)
        );
	}
        
    // we only take the first one (there should only be one)
    $key   = array_key_first( $hit );
    $value = intval( $hit[ $key ] );

	// lets get the result from the database
	$psql1 = $wpdb->prepare( "select result from {$wpdb->prefix}lazy_captcha where nid='%s' and chtime > (CURRENT_TIMESTAMP() - 3600*12);", substr($key,0,18) );
    $res1  = $results = $wpdb->get_results($psql1);
    
    $result = "";
    if ($res1) {
		$result = $res1[0]->result;
	} 
    
	// did we get an incorrect result
	if ( (int) $result !== $value) {
		wp_die( 
			apply_filters('pre_comment_content', 
			"<strong>" . __( 'Error', 'wp-lazycaptcha' ) . '</strong>:' . __( 'Please fill in the correct answer and try again.', 'wp-lazycaptcha' ) ),
			'Error',
			array('back_link' => true)
        );
	}
  
    // delete captcha record from database if everything was okay
    $psql2= $wpdb->prepare( "delete from {$wpdb->prefix}lazy_captcha where nid='%s';", substr($key,0,18));
    $res2 = $wpdb->query($psql2);  
    
    return $comment;
}

/* ----------------- M A I N --------------- */

// activating deactivating the plugin
register_activation_hook(__FILE__,'wp_lazy_captcha_activate');
register_deactivation_hook(__FILE__,'wp_lazy_captcha_deactivate');

// init plugin
add_action('init', 'wp_lazy_captcha_init');
// add captcha field to comment form
add_filter( 'comment_form_defaults', 'wp_lazy_captcha_comment_meta' );
// add action if comment is submitted
add_filter('preprocess_comment', 'wp_lazy_captcha_check', 0, 1);
